require 'spec_helper'

describe "Keeping only Helix TeamHub users in sync" do

  def initialize_dependencies(reader='ldap')
    @rest_manager = stub_everything('rest_manager')
    @source_reader = stub('ldap_reader')
    @rest_manager.stubs(:find_all).with(only_active: true).returns({status: 200, content: @hth_users})
    @syncer_options = {
      apply: true,
      reader: reader,
      source: :test_source
    }
  end

  let(:syncher) do
    HTH::UserSyncher.new(
      @syncer_options,
      ldap_reader: @source_reader,
      rest_user_manager: @rest_manager
    )
  end

  before do
    @user1 = build_user
    @user2 = build_user
  end

  context "finding users from Helix TeamHub" do
    before(:each) do
      @hth_users = @source_users = []
      initialize_dependencies()
      @source_reader.stubs(:find_users_by_ids).
        with([]).returns([])
    end

    it "requests only active users" do
      @rest_manager.expects(:find_all).with(only_active: true).
        returns({status: 200, content: @hth_users})
      syncher.keep_in_sync
    end
  end

  context "when Helix TeamHub and source users are equal" do
    before do
      @hth_users = [@user1, @user2]
      initialize_dependencies()
      @source_reader.stubs(:find_users_by_ids).
        with(@hth_users.map {|u| u[:id] }).returns(@hth_users)
    end

    it "updates existing users" do
      @rest_manager.expects(:batch_update).with([@user1, @user2]).once
      syncher.keep_in_sync
    end

    it "does not delete users" do
      @rest_manager.expects(:batch_delete).never
      syncher.keep_in_sync
    end
  end

  context "when Helix TeamHub user has been deleted from source" do
    before do
      @hth_users  = [@user1, @user2]
      initialize_dependencies()
      @source_reader.stubs(:find_users_by_ids).
        with(@hth_users.map {|u| u[:id] }).returns([@user1])
    end

    it "updates existing users" do
      @rest_manager.expects(:batch_update).with([@user1]).once
      syncher.keep_in_sync
    end

    it "removes deleted users" do
      @rest_manager.expects(:batch_delete).with([@user2[:id]]).once
      syncher.keep_in_sync
    end

    context "with force-sync" do
      before do
        @syncer_options = @syncer_options.merge(force_sync: true)
      end

      it "updates existing users" do
        @rest_manager.expects(:batch_update).with([@user1]).once
        syncher.keep_in_sync
      end

      it "does not delete users" do
        @rest_manager.expects(:batch_delete).never
        syncher.keep_in_sync
      end
    end
  end

  context "when Helix TeamHub user has been deactivated from source" do
    [true, false].each do |bool|
      context "when force-sync is #{bool ? 'enabled' : 'disabled'}" do
        before do
          @hth_users  = [@user1, @user2]
          initialize_dependencies()
          @source_reader.stubs(:find_users_by_ids).
            with(@hth_users.map {|u| u[:id] }).
            returns([@user1, deactivate_source_user(@user2)])
          @syncer_options = @syncer_options.merge(force_sync: bool)
        end

        it "updates existing users" do
          @rest_manager.expects(:batch_update).with([@user1]).once
          syncher.keep_in_sync
        end

        it "removes deactivated users" do
          @rest_manager.expects(:batch_delete).with([@user2[:id]]).once
          syncher.keep_in_sync
        end
      end
    end
  end
end
