require 'spec_helper'

describe HTH::UserSyncher do
  context "#get_hth_user" do
    let(:syncher)     { HTH::UserSyncher.new({apply: true, reader: 'ldap'}) }
    let(:hth_users) { [build_user, deactivate_hth_user(build_user)] }

    it "returns nil without users" do
      syncher.send(:get_hth_user, [], 'foo').should be_nil
    end

    it "returns nil without matches" do
      syncher.send(:get_hth_user, hth_users, 'foo').should be_nil
    end

    it "returns nil when identifier is nil" do
      syncher.send(:get_hth_user, hth_users, nil).should be_nil
    end

    it "returns user when found by id" do
      syncher.send(:get_hth_user, hth_users, hth_users.first[:short_name]).should eq hth_users.first
    end

    it "does not return user by old_id" do
      syncher.send(:get_hth_user, hth_users, hth_users.last[:short_name]).should be_nil
    end
  end

  context "#add_source" do
    let(:config) {{ source: 'example_ldap' }}
    let(:syncher) { HTH::UserSyncher.new({ apply: true, reader: 'ldap', source: config[:source] }) }

    it "adds source identifier from reader options" do
      hash = syncher.send(:add_source, {})
      hash[:source].should eq config[:source]
    end
  end

  context "#filter_users_by_source" do
    before do
      @options = { apply: true, reader: 'ldap', source: :valid_source }

      @hth_users = [
        build_user.merge(source: :bar),
        build_user.merge(source: "")
      ]
      3.times do |i|
        @hth_users << build_user.merge(source: :valid_source)
      end
    end

    let(:syncher) { HTH::UserSyncher.new(@options) }

    it "filters out users without matching source" do
      syncher.send(:filter_users_by_source, @hth_users).size.should eq 3
    end

    context "with force_sync set to true" do
      before do
        @options.merge!(force_sync: true)
      end

      it "does no filtering" do
        syncher.send(:filter_users_by_source, @hth_users).size.should eq 5
      end
    end
  end

  context "#hth_users_no_longer_in_source" do
    before do
      @options = { apply: true, reader: 'ldap', source: :test_source }

      user1 = build_user
      user2 = build_user
      user3 = build_user

      @hth_users = [
        user1,
        user2,
        deactivate_hth_user(user3)
      ]
      @source_users = [
        user1
      ]
    end

    let(:syncher) { HTH::UserSyncher.new(@options) }

    it "returns active hth users not present in the source" do
      result = syncher.send(:hth_users_no_longer_in_source, @hth_users, @source_users)
      result.should eq %w(ckent2)
    end

    context "with force_sync set to true" do
      before do
        @options.merge!(force_sync: true)
      end

      it "returs empty array" do
        result = syncher.send(:hth_users_no_longer_in_source, @hth_users, @source_users)
        result.should be_empty
      end
    end
  end
end
